//    
//  
// antony@nline.ru

#include "asmegam.h"
#include "ADC.h"

void ADC_Init_10bit(unsigned char channel, ADC_PRESCALER prescaler, unsigned char internalref)
{
	ADCSRA = 0x00; // Disable ADC
	_NOP();
	ADCSRA |= (1<<ADIF); // Clear conversion complete flag
	_NOP();
	
	if(!internalref)
		ADMUX = channel | 0b01000000; // AVCC as reference, right-adjusted format
	else
		ADMUX = channel | 0b11000000; // internal reference, right-adjusted format
	
	ADCSRA = 0b10000000 | prescaler; // Enable ADC, perform initialisation
	
	_NOP();
	
	ADCSRA |= (1<<ADSC); // Start conversion
	
	while (!(ADCSRA&(1<<ADIF))) // Wait till conversion is complete
		;
	
	ADCSRA |= (1<<ADIF); // Clear conversion complete flag
	_NOP(); 
	ADCSRA |= (1<<ADIF); // Clear conversion complete flag
}

unsigned int ADC_GetValue(void)
{
	ADCSRA |= (1<<ADIF); // Clear conversion complete flag
	_NOP(); 
	ADCSRA |= (1<<ADIF); // Clear conversion complete flag 
	_NOP();
	ADCSRA &= ~(1<<ADIE); // Interrupt disabled
	_NOP();
	ADCSRA |= (1<<ADSC); // Start conversion
	_NOP();
	
	while(!(ADCSRA&(1<<ADIF))) // Wait till conversion is complete
		;
	
	return ADC;
}

#pragma interrupt_handler adc_isr:22
void adc_isr(void)
{
	// empty, used only to wake up
}

unsigned int ADC_GetValue_NR(void)
{
	unsigned char temp;
	
	ADCSRA |= (1<<ADIF); // Clear conversion complete flag
	_NOP(); 
	ADCSRA |= (1<<ADIF); // Clear conversion complete flag 
	_NOP();
	ADCSRA &= ~(1<<ADIE); // Interrupt disabled
	_NOP();
	ADCSRA |= (1<<ADSC); // Start conversion
	_NOP();
	
	do
	{
		temp = MCUCR; // adc nr sleep mode
		temp |= (1<<SE);
		temp |= (1<<SM0);
		temp &= ~(1<<SM1);
		MCUCR = temp;
		SEI();
		asm("sleep");
	}
	while(!(ADCSRA & (1<<ADIF)));
	
	return ADC; // Return ADC registers
}
